(ns org.aurellem.worm-learn
  "General worm creation framework."
  {:author "Robert McIntyre"}
  (:use (cortex world util import body sense
                hearing touch vision proprioception movement))
  (:import (com.jme3.math ColorRGBA Vector3f))
  (:import java.io.File)
  (:import com.jme3.audio.AudioNode)
  (:import com.aurellem.capture.RatchetTimer)
  (:import (com.aurellem.capture Capture IsoTimer))
  (:import (com.jme3.math Vector3f ColorRGBA)))
  

(dorun (cortex.import/mega-import-jme3))
(rlm.rlm-commands/help)


(def hand "Models/test-creature/hand.blend")

(defn worm-model []
  (load-blender-model "Models/worm/worm.blend"))

(def output-base (File. "/home/r/proj/cortex/render/worm-learn/"))


(defn motor-control-program
  "Create a function which will execute the motor script"
  [muscle-positions
   script]
  (let [current-frame (atom -1)
        keyed-script (group-by first script)
        current-forces (atom {}) ]
    (fn [effectors]
      (let [indexed-effectors (vec effectors)]
        (dorun 
         (for [[_ part force] (keyed-script (swap! current-frame inc))]
           (swap! current-forces (fn [m] (assoc m part force)))))
        (doall (map (fn [effector power]
                      (effector (int power)))
                    effectors
                    (map #(@current-forces % 0) muscle-positions)))))))

(def muscle-labels
  [:base-up :base-down
   :a-up :a-down
   :b-up :b-down
   :c-up :c-down
   :d-up :d-down
   ])

(def curl-script
  [[370 :d-up 20]
   [390 :d-up 0]])

(def period 18)

(defn gen-wiggle [[flexor extensor :as muscle-pair] time-base]
  (let [period period
        power 45]
    [[time-base flexor power]
     [(+ time-base period) flexor 0]
     [(+ time-base period 1) extensor power]
     [(+ time-base (+ (* 2 period) 2))  extensor 0]]))
  
(def wiggle-script
  (mapcat gen-wiggle [[:d-up :d-down]
                      [:c-up :c-down]
                      [:b-up :b-down]
                      [:a-up :a-down]] (range 100 1000 12)))

(def wiggle-script
  (mapcat gen-wiggle (repeat 40 [:a-down :a-up])
                     (range 100 10000 (+ 3 (* period 2)))))



(defn worm-world
  ""
  ([] (worm-world curl-script))
  ([motion-script]
     (let [record? false
           worm (doto (worm-model) (body!))
           touch   '();;(touch! worm)
           prop    (proprioception! worm)
           muscles (movement! worm)
           
           touch-display  (view-touch)
           prop-display   (view-proprioception)
           muscle-display (view-movement)

           floor (box 10 1 10 :position (Vector3f. 0 -10 0)
                      :color ColorRGBA/Gray :mass 0)

           control-script (motor-control-program
                           muscle-labels motion-script)]
       (world
        (nodify [worm floor])
        standard-debug-controls
        
        (fn [world]
          ;; (set-gravity world Vector3f/ZERO)
          ;; (position-camera
          ;;  world (Vector3f. 4.207176, -3.7366982, 3.0816958)
          ;;  (Quaternion. 0.11118768, 0.87678415, 0.24434438, -0.3989771))



          (position-camera
           world (Vector3f. 4.207176, -3.7366982, 3.0816958)
           (Quaternion. -0.11555642, 0.88188726, -0.2854942, -0.3569518))




          (let [timer (IsoTimer. 60)]
            (.setTimer world timer)
            (display-dilated-time world timer))
          (if record?
            (Capture/captureVideo
             world
             (File. output-base "main-view")))
          (speed-up world)
          (light-up-everything world))
        
        (fn [world tpf]
          (muscle-display
           (control-script muscles)
           (if record? (File. output-base "muscle")))
          (prop-display
           (prop)
           (if record? (File. output-base "proprio")))             
          (touch-display 
           (map #(% (.getRootNode world)) touch)
           (if record?
             (File. output-base "touch"))))))))
